#****************************************************************************
#**
#**  File     :  /cdimage/units/URA0104/URA0104_script.lua
#**  Author(s):  John Comes, David Tomandl
#**
#**  Summary  :  Cybran T2 Air Transport Script
#**
#**  Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local CAirUnit = import('/lua/cybranunits.lua').CAirUnit
local explosion = import('/lua/defaultexplosions.lua')
local weaponfile2 = import('/mods/HawksmodFA/lua/Hawkeyeweapons.lua')
local util = import('/lua/utilities.lua')
local Weapon = import('/lua/sim/Weapon.lua').Weapon
local cWeapons = import('/lua/cybranweapons.lua')
local CDFLaserHeavyWeapon = cWeapons.CDFLaserHeavyWeapon
local CDFParticleCannonWeapon = cWeapons.CDFParticleCannonWeapon
local MartyrHeavyMicrowaveLaserGenerator = weaponfile2.MartyrHeavyMicrowaveLaserGenerator
local CAAAutocannon = cWeapons.CAAAutocannon
local CEMPAutoCannon = cWeapons.CEMPAutoCannon

URA0104 = Class(CAirUnit) {
    DestroyNoFallRandomChance = 1.1,
    Weapons = {
        MainGun = Class(MartyrHeavyMicrowaveLaserGenerator) {},
        LeftPD01 = Class(CDFLaserHeavyWeapon) {},
        LeftPD02 = Class(CDFLaserHeavyWeapon) {},
        LeftPD03 = Class(CDFLaserHeavyWeapon) {},
        RightPD01 = Class(CDFLaserHeavyWeapon) {},
        RightPD02 = Class(CDFLaserHeavyWeapon) {},
        RightPD03 = Class(CDFLaserHeavyWeapon) {},
        UnderGun = Class(CDFParticleCannonWeapon) {},
        AAGun01 = Class(CAAAutocannon) {
            FxMuzzleScale = 2.25,
        },
        AAGun02 = Class(CAAAutocannon) {
            FxMuzzleScale = 2.25,
        },
        MiddleAA01 = Class(CAAAutocannon) {},
        MiddleAA02 = Class(CAAAutocannon) {},
        LeftAAGun01 = Class(CAAAutocannon) {},
        LeftAAGun02 = Class(CAAAutocannon) {},
        RightAAGun01 = Class(CAAAutocannon) {},
        RightAAGun02 = Class(CAAAutocannon) {},
        BackAAGun01 = Class(CAAAutocannon) {},
        BackAAGun02 = Class(CAAAutocannon) {},
    },

    AirDestructionEffectBones = { 'Left_Exhaust', 'Right_Exhaust', 'Char04', 'Char03', 'Char02', 'Char01',
                                  'Front_Left_Leg03_B02', 'Front_Right_Leg03_B02', 'Front_Left_Leg01_B02', 'Front_Right_Leg01_B02',
                                  'Right_AttachPoint01', 'Right_AttachPoint02', 'Right_AttachPoint03', 'Right_AttachPoint04',
                                  'Left_AttachPoint01', 'Left_AttachPoint02', 'Left_AttachPoint03', 'Left_AttachPoint04', },

    BeamExhaustIdle = '/effects/emitters/missile_exhaust_fire_beam_05_emit.bp',
    BeamExhaustCruise = '/effects/emitters/missile_exhaust_fire_beam_04_emit.bp',
   


    # When one of our attached units gets killed, detach it
    OnAttachedKilled = function(self, attached)
        attached:DetachFrom()
    end,

    OnKilled = function(self, instigator, type, overkillRatio)
        CAirUnit.OnKilled(self, instigator, type, overkillRatio)
        # TransportDetachAllUnits takes 1 bool parameter. If true, randomly destroys some of the transported
        # units, otherwise successfully detaches all.
        self:TransportDetachAllUnits(true)
    end,


    # Override air destruction effects so we can do something custom here
    CreateUnitAirDestructionEffects = function( self, scale )
        self:ForkThread(self.AirDestructionEffectsThread, self )
    end,

    AirDestructionEffectsThread = function( self )
        local numExplosions = math.floor( table.getn( self.AirDestructionEffectBones ) * 0.5 )
        for i = 0, numExplosions do
            explosion.CreateDefaultHitExplosionAtBone( self, self.AirDestructionEffectBones[util.GetRandomInt( 1, numExplosions )], 0.5 )
            WaitSeconds( util.GetRandomFloat( 0.2, 0.9 ))
        end
    end,
}

TypeClass = URA0104

