#****************************************************************************
#**
#**  File     :  /cdimage/units/UEB5202/UEB5202_script.lua
#**  Author(s):  John Comes, David Tomandl
#**
#**  Summary  :  UEF Air Staging Platform
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local TPodTowerUnit = import('/lua/terranunits.lua').TPodTowerUnit
local TANTorpedoLandWeapon = import('/lua/terranweapons.lua').TANTorpedoLandWeapon
local TAAFlakArtilleryCannon = import('/lua/terranweapons.lua').TAAFlakArtilleryCannon



XEB5205 = Class(TPodTowerUnit) {    
    Weapons = {
        FlakGun01 = Class(TAAFlakArtilleryCannon) {},
        FlakGun02 = Class(TAAFlakArtilleryCannon) {},
        FlakGun03 = Class(TAAFlakArtilleryCannon) {},
        FlakGun04 = Class(TAAFlakArtilleryCannon) {},
        TorpedoTurret01 = Class(TANTorpedoLandWeapon) {},
        TorpedoTurret02 = Class(TANTorpedoLandWeapon) {},
        TorpedoTurret03 = Class(TANTorpedoLandWeapon) {},
        TorpedoTurret04 = Class(TANTorpedoLandWeapon) {},
           
    },

    ShieldEffects = {
        '/effects/emitters/terran_shield_generator_t2_01_emit.bp',
        '/effects/emitters/terran_shield_generator_t2_02_emit.bp',
        ###'/effects/emitters/terran_shield_generator_t2_03_emit.bp',
    },
    
    OnStopBeingBuilt = function(self,builder,layer)
        TPodTowerUnit.OnStopBeingBuilt(self,builder,layer)
        if not self.OpenAnim then
            self.OpenAnim = CreateAnimator(self)
            self.Trash:Add(self.OpenAnim)
            self.Rotator1 = CreateRotator(self, 'Spinner', 'y', nil, 10, 5, 10)
            self.Rotator2 = CreateRotator(self, 'B01', 'z', nil, -10, 5, -10)
            self.Trash:Add(self.Rotator1)
            self.Trash:Add(self.Rotator2)
		    self.ShieldEffectsBag = {}
#		    self.EnabledResourceDrains = {
#            Shield = false,
#            Intel = false,
#            } 
            
#            self:SetScriptBit('RULEUTC_ShieldToggle', true)
#            self:SetScriptBit('RULEUTC_IntelToggle', true)
        end
        self.OpenAnim:PlayAnim(self:GetBlueprint().Display.AnimationOpen, false):SetRate(0.4)
#        TPodTowerUnit.OnStopBeingBuilt(self,builder,layer)
    end,
    
     OnShieldEnabled = function(self)
        TPodTowerUnit.OnShieldEnabled(self)
        if self.Rotator1 then
            self.Rotator1:SetTargetSpeed(10)
        end
        if self.Rotator2 then
            self.Rotator2:SetTargetSpeed(-10)
        end
        
        if self.ShieldEffectsBag then
            for k, v in self.ShieldEffectsBag do
                v:Destroy()
            end
		    self.ShieldEffectsBag = {}
		end
        for k, v in self.ShieldEffects do
            table.insert( self.ShieldEffectsBag, CreateAttachedEmitter( self, 'Shield_Base', self:GetArmy(), v ) )
        end
    end,

    OnShieldDisabled = function(self)
        TPodTowerUnit.OnShieldDisabled(self)
        self.Rotator1:SetTargetSpeed(0)
        self.Rotator2:SetTargetSpeed(0)
        
        if self.ShieldEffectsBag then
            for k, v in self.ShieldEffectsBag do
                v:Destroy()
            end
		    self.ShieldEffectsBag = {}
		end
    end,
    
    SpawnPOD = function(self)
    if not self:IsDead() then

        ### Gets the platforms current orientation
        local platOrient = self:GetOrientation()
           
        ### Randomly generates a position for the POD / Drone to face after spawning
        #Degrees +/- centre
        local degrees = 45
        #Get random angle, convert it to radians
        local angle = (180 + math.random(degrees * 2) - degrees) * (math.pi / 180)
        #Do the math
        local x = math.cos(angle / 2)
        local z = math.sin(angle / 2) 
        local droneOrient = x
     
        ### Gets the current position of the Kennel in the game world
        local location = self:GetPosition('xeb5205')

        ### Creates our drone over the kennel with a ranomly generated orientation
        local drone = CreateUnit('XEA3309', self:GetArmy(), location[1], location[2], location[3], platOrient[1], droneOrient, platOrient[3], platOrient[4], 'Air')

         ### Sets the Kennel unit as the drones parent
         drone:SetParent(self, 'xeb5205')
         drone:SetCreator(self) 

         ### The drone will guard the parent kennel unit directly
         IssueClearCommands({drone})
         IssueGuard({drone}, self)

         ###drone clean up scripts
         self.Trash:Add(drone)

    end
end, 
}


TypeClass = XEB5205