#****************************************************************************
#**
#**  File     :  /data/units/XSB0405/XSB0405_script.lua
#**  Author(s):  Jessica St. Croix, Greg Kohne
#**
#**  Summary  :  Seraphim T3 Power Generator Script
#**
#**  Copyright  2007 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
local SStructureUnit = import('/lua/seraphimunits.lua').SStructureUnit
local WeaponsFile = import ('/mods/HawksmodFA/lua/Hawkeyeweapons.lua')
local LambdaWeapon = WeaponsFile.LambdaWeapon
local SSeraphimSubCommanderGateway01 = import('/lua/EffectTemplates.lua').SeraphimSubCommanderGateway01
local SSeraphimSubCommanderGateway02 = import('/lua/EffectTemplates.lua').SeraphimSubCommanderGateway02
local SSeraphimSubCommanderGateway03 = import('/lua/EffectTemplates.lua').SeraphimSubCommanderGateway03
local LambdaField = import('/mods/HawksmodFA/lua/Hawkeyedefaultantiprojectile.lua').LambdaField
local SeraLambdaField = import('/mods/HawksmodFA/lua/Hawkeyedefaultantiprojectile.lua').SeraLambdaField
local utilities = import('/lua/utilities.lua')
local EffectUtil = import('/lua/EffectUtilities.lua')
local explosion = import('/lua/defaultexplosions.lua')

XSB0405 = Class(SStructureUnit) {

	SpawnEffects = {
		'/effects/emitters/seraphim_othuy_spawn_01_emit.bp',
		'/effects/emitters/seraphim_othuy_spawn_02_emit.bp',
		'/effects/emitters/seraphim_othuy_spawn_03_emit.bp',
		'/effects/emitters/seraphim_othuy_spawn_04_emit.bp',
	},
    AmbientEffects = 'ST3PowerAmbient',
        Weapons = {
        Eye01 = Class(LambdaWeapon) {},
        Eye02 = Class(LambdaWeapon) {},
        },
    
    OnStopBeingBuilt = function(self, builder, layer)
        for k, v in SSeraphimSubCommanderGateway01 do
            CreateAttachedEmitter(self, 'Spinner', self:GetArmy(), v)
            CreateAttachedEmitter(self, 'Light04', self:GetArmy(), v)
            CreateAttachedEmitter(self, 'Light05', self:GetArmy(), v)
            CreateAttachedEmitter(self, 'Light06', self:GetArmy(), v)
#            CreateAttachedEmitter(self, 'Eye01', self:GetArmy(), v)
        end
        for k, v in SSeraphimSubCommanderGateway02 do
            CreateAttachedEmitter(self, 'Light01', self:GetArmy(), v)
            CreateAttachedEmitter(self, 'Light02', self:GetArmy(), v)
            CreateAttachedEmitter(self, 'Light03', self:GetArmy(), v)
        end
        SStructureUnit.OnStopBeingBuilt(self, builder, layer)
        self.Trash:Add(CreateRotator(self, 'Orb01', 'y', nil, 0, 15, 80 + Random(0, 20)))
        self.Trash:Add(CreateRotator(self, 'Orb02', 'y', nil, 0, 15, 80 + Random(0, 20)))
        self.Trash:Add(CreateRotator(self, 'Spinner', 'y', nil, 0, 10, 30 + Random(0, 10)))
        self:SetMaintenanceConsumptionActive()
        local bp = self:GetBlueprint().Defense.SeraLambdaField01
        local bp2 = self:GetBlueprint().Defense.SeraLambdaField02
        local bp3 = self:GetBlueprint().Defense.SeraLambdaField03
        local SeraLambdaField01 = SeraLambdaField {
            Owner = self,
            Radius = bp.Radius,
            AttachBone = bp.AttachBone,
            RedirectRateOfFire = bp.RedirectRateOfFire
        }
        local SeraLambdaField02 = SeraLambdaField {
            Owner = self,
            Radius = bp2.Radius,
            AttachBone = bp2.AttachBone,
            RedirectRateOfFire = bp2.RedirectRateOfFire
        }
        local SeraLambdaField03 = SeraLambdaField {
            Owner = self,
            Radius = bp3.Radius,
            AttachBone = bp3.AttachBone,
            RedirectRateOfFire = bp3.RedirectRateOfFire
        }
        self.Trash:Add(SeraLambdaField01)
        self.Trash:Add(SeraLambdaField02)
        self.Trash:Add(SeraLambdaField03)
        self.UnitComplete = true
    end,
    
#    	OnKilled = function(self)
#        self.Trash:Add(CreateRotator(self, 'Spinner', 'y', nil, 0, 0, 0 + Random(0, 0)))
#				
#        SStructureUnit.OnKilled(self)
#    end,
    
    DeathThread = function( self, overkillRatio , instigator)
        local bigExplosionBones = {'Spinner', 'Eye01', 'Eye02'}
        local explosionBones = {'XSB0405', 'Light01',
                                'Light02', 'Light03',
                                'Light04', 'Light05', 'Light06',
                                }
                                        
        explosion.CreateDefaultHitExplosionAtBone( self, bigExplosionBones[Random(1,3)], 4.0 )
        explosion.CreateDebrisProjectiles(self, explosion.GetAverageBoundingXYZRadius(self), {self:GetUnitSizes()})           
        WaitSeconds(2)
        
        local RandBoneIter = RandomIter(explosionBones)
        for i=1,Random(4,6) do
            local bone = RandBoneIter()
            explosion.CreateDefaultHitExplosionAtBone( self, bone, 1.0 )
            WaitTicks(Random(1,3))
        end
        
        local bp = self:GetBlueprint()
        for i, numWeapons in bp.Weapon do
            if(bp.Weapon[i].Label == 'CollossusDeath') then
                DamageArea(self, self:GetPosition(), bp.Weapon[i].DamageRadius, bp.Weapon[i].Damage, bp.Weapon[i].DamageType, bp.Weapon[i].DamageFriendly)
                break
            end
        end
        WaitSeconds(3.5)
        explosion.CreateDefaultHitExplosionAtBone( self, 'Spinner', 5.0 )        

        if self.DeathAnimManip then
            WaitFor(self.DeathAnimManip)
        end

    
        self:DestroyAllDamageEffects()
        self:CreateWreckage( overkillRatio )

        # CURRENTLY DISABLED UNTIL DESTRUCTION
        # Create destruction debris out of the mesh, currently these projectiles look like crap,
        # since projectile rotation and terrain collision doesn't work that great. These are left in
        # hopes that this will look better in the future.. =)
        if( self.ShowUnitDestructionDebris and overkillRatio ) then
            if overkillRatio <= 1 then
                self.CreateUnitDestructionDebris( self, true, true, false )
            elseif overkillRatio <= 2 then
                self.CreateUnitDestructionDebris( self, true, true, false )
            elseif overkillRatio <= 3 then
                self.CreateUnitDestructionDebris( self, true, true, true )
            else #VAPORIZED
                self.CreateUnitDestructionDebris( self, true, true, true )
            end
        end

        
        # Spawn an engineer (temp energy being)
        local position = self:GetPosition()
        local spiritUnit = CreateUnitHPR('XSL0403', self:GetArmy(), position[1], position[2], position[3], 0, 0, 0)
        
        # Create effects for spawning of energy being
		for k, v in self.SpawnEffects do
			CreateAttachedEmitter(spiritUnit, -1, self:GetArmy(), v )
		end	
        
        self:PlayUnitSound('Destroyed')
        self:Destroy()
    end,
}

TypeClass = XSB0405